@@ -3,7 +3,6 @@ import copy
 import numpy as np
 from typing import Dict
 
-DEFAULT_DATA = {"instance_score": None}  # type: Dict
 DEFAULT_META = {
     "name": None,
     "detector_type": None,  # online or offline
@@ -12,32 +11,39 @@ DEFAULT_META = {
 
 
 def outlier_prediction_dict():
-    data = DEFAULT_DATA
-    data['feature_score'] = None
-    data['is_outlier'] = None
+    data = {
+        'instance_score': None,
+        'feature_score': None,
+        'is_outlier': None
+    }
     return copy.deepcopy({"data": data, "meta": DEFAULT_META})
 
 
 def adversarial_prediction_dict():
-    data = DEFAULT_DATA
-    data['is_adversarial'] = None
+    data = {
+        'instance_score': None,
+        'is_adversarial': None
+    }
     return copy.deepcopy({"data": data, "meta": DEFAULT_META})
 
 
 def adversarial_correction_dict():
-    data = DEFAULT_DATA
-    data['is_adversarial'] = None
-    data['corrected'] = None
-    data['no_defense'] = None
-    data['defense'] = None
+    data = {
+        'instance_score': None,
+        'is_adversarial': None,
+        'corrected': None,
+        'no_defense': None,
+        'defense': None
+    }
     return copy.deepcopy({"data": data, "meta": DEFAULT_META})
 
 
 def concept_drift_dict():
     data = {
-        "batch_score": None,
-        "feature_score": None,
-        "is_drift": None
+        'is_drift': None,
+        'distance': None,
+        'p_val': None,
+        'threshold': None
     }
     return copy.deepcopy({"data": data, "meta": DEFAULT_META})
 
